/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT %Version%
 *
 */

package EDU.rutgers.CORBA.GIOP.messages;

import ORG.omg.CORBA.OctetSequence;

import EDU.rutgers.act.presentation.TypeReader;
import EDU.rutgers.act.presentation.TypeWriter;

/**
 * [CORBA 2.0 Specification 12-21]
 *
 * @see GIOPMessage
 *
 * @see GIOPRequestMessage
 * @see GIOPReplyMessage
 * @see GIOPCancelRequestMessage
 * @see GIOPLocateReplyMessage
 * @see GIOPCloseConnectionMessage
 * @see GIOPMessageErrorMessage
 *
 * @since ACT 0.1
 */
public final class GIOPLocateRequestMessage extends GIOPMessage
{
    /**
     * Identifies the object being located
     *
     * @since ACT 0.1
     */
	public byte[] object_key_;

    private int locateRequestHeaderSize_;         // network size
    
    /**
     * Empty constructor, it could only be used by the GIOPMessageFactory
     *
     * @since ACT 0.1
     */
    protected GIOPLocateRequestMessage() {
        super(MessageType.LocateRequest);
    }

    /**
     * @param int request_id
     * @param byte[] object_key
     * 
     * @since ACT 0.1
     */
    public GIOPLocateRequestMessage(
        int request_id,
        byte[] object_key) {
        super(MessageType.LocateRequest);

        request_id_ = request_id;
        object_key_ = object_key;

        calLocateRequestHeaderSize();
        super.setMessageSize(locateRequestHeaderSize_);
    }

    /**
     * Calculate the header network size. In case the user just resuse this object
     * by replacing some member fields, this method forces to reset the GIOP header
     * message_size field.
     *
     * @since ACT 0.1
     */
    public void calLocateRequestHeaderSize() {
        int twoHeadersSize = super.giopHeaderSize_;
        twoHeadersSize += super.ts_.calAlignGap(twoHeadersSize, 4) + 4; // request_id

        // object_key_.length as unsigned long
        twoHeadersSize += super.ts_.calAlignGap(twoHeadersSize, 4) + 4;
        int len = object_key_.length;
        twoHeadersSize += super.ts_.calAlignGap(twoHeadersSize, 1) + len; // object_key_

        locateRequestHeaderSize_ = twoHeadersSize - super.giopHeaderSize_;

        super.setMessageSize(locateRequestHeaderSize_);
    }

    /**
     * Read from the TypeReader stream
     *
     * @exception java.io.IOException If any IO error occurs
     *
     * @see ORG.omg.CORBA.Streamable#read
     * @see EDU.rutgers.act.presentation.TypeReader
     *
     * @since ACT 0.1
     */
    public void read(TypeReader in) 
		throws java.io.IOException {
        super.read(in);

		request_id_ = in.readULong();
		int len = in.readULong();
		object_key_ = new byte[len];
		in.readOctetArray(object_key_, 0, len);
	}

    /**
     * Write to the TypeWriter stream.
     *
     * @exception java.io.IOException If any IO error occurs
     *
     * @see ORG.omg.CORBA.Streamable#write
     * @see EDU.rutgers.act.presentation.TypeWriter
     *
     * @since ACT 0.1
     */
    public void write(TypeWriter out)
		throws java.io.IOException {
        super.write(out);

		out.writeULong(request_id_);
		int len = object_key_.length;
		out.writeULong(len);
		out.writeOctetArray(object_key_, 0, len);
	}

// Object
/*
    public boolean equals(Object o) {
		if(super.equals(o) && (o != null) && (o instanceof GIOPLocateRequestMessage)) {
            if (o == this) {
                return true;
            }
            GIOPLocateRequestMessage msg = (GIOPLocateRequestMessage)o;
            if (request_id_ == msg.request_id_ && 
                String.valueOf(object_key_).equals(String.valueOf(msg.object_key_))) {
                return true;
            }
        }
        return false;
    }
*/

    /**
     * Returns a string representation. 
     *
     * @return String description
     *
     * @since ACT 0.1
     */
    public String toString() {
        StringBuffer buf = new StringBuffer(super.toString()).
            append("\nGIOP:LocateRequestMessage").
            append("\n[Header] ").
            append("\n\trequest_id: ").
            append(request_id_).
            append("\n\tobject_key: ").append(OctetSequence.byteArray2String(object_key_)).
            append("\n[Body]").append("None");
        return String.valueOf(buf);
    }
}
