/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT %Version%
 *
 */

package EDU.rutgers.CORBA.GIOP.messages;

import ORG.omg.CORBA.BAD_PARAM;

/**
 * [CORBA 2.0 Specification 12-19]
 *
 * @since ACT 0.1
 */
public final class ReplyStatusType
{
    /**
     * NO_EXCEPTION type value
     *
     * @since ACT 0.1
     */
	public static final int _NO_EXCEPTION = 0;

    /**
     * USER_EXCEPTION type value
     *
     * @since ACT 0.1
     */
    public static final int _USER_EXCEPTION = 1;

    /**
     * SYSTEM_EXCEPTION type value
     *
     * @since ACT 0.1
     */
    public static final int _SYSTEM_EXCEPTION = 2;

    /**
     * LOCATION_FORWARD type value
     *
     * @since ACT 0.1
     */
    public static final int _LOCATION_FORWARD = 3;

    /**
     * NO_EXCEPTION type : 
     * Indicates the body is encoded as if it were a structure 
     * holding the first any operation value, then any inout and out parameters 
     * in the order in which they appear in the operation's OMG IDL definition, 
     * from left to right
     *
     * @since ACT 0.1
     */
    public static final ReplyStatusType NO_EXCEPTION = 
		new ReplyStatusType(_NO_EXCEPTION);

    /**
     * USER_EXCEPTION type :
     * The body contains the exception that was raised by the operation, encoded as exception
     *
     * @since ACT 0.1
     */
    public static final ReplyStatusType USER_EXCEPTION =
		new ReplyStatusType(_USER_EXCEPTION);

    /**
     * SYSTEM_EXCEPTION type :
     *
     * The body contains the exception that was raised by the operation, encoded as exception
     *
     * @since ACT 0.1
     */
    public static final ReplyStatusType SYSTEM_EXCEPTION = 
		new ReplyStatusType(_SYSTEM_EXCEPTION);

    /**
     * LOCATION_FORWARD type :
     * The body contains an object reference (IOR).
     *
     * @see EDU.rutgers.act.IOP.IOR
     * 
     * @since ACT 0.1
     */
    public static final ReplyStatusType LOCATION_FORWARD =
		new ReplyStatusType(_LOCATION_FORWARD);

    private ReplyStatusType(int v) {
        value = v;
    }

    private int value;

    /**
     * Retrieve the type value
     *
     * @return int the type value
     *
     * @since ACT %Version%
     */
	public int value() {
		return value;
	}

    /**
     * From the type integer value, retrieve the ReplyStatusType
     *
     * @param int the type integer value
     *
     * @return ReplyStatusType the corresponding ReplyStatusType
     *
     * @exception BAD_PARAM if the integer value is out of range
     *
     * @see ORG.omg.CORBA.BAD_PARAM
     *
     * @since ACT 0.1
     */
    public static ReplyStatusType type(int v) 
		throws BAD_PARAM {
		switch(v) {
			default:
				throw new BAD_PARAM("Enum out of range [0...3] " + v);

			case _NO_EXCEPTION:
				return ReplyStatusType.NO_EXCEPTION;

			case _USER_EXCEPTION:
				return ReplyStatusType.USER_EXCEPTION;

			case _SYSTEM_EXCEPTION:
				return ReplyStatusType.SYSTEM_EXCEPTION;

			case _LOCATION_FORWARD:
				return ReplyStatusType.LOCATION_FORWARD;
        }
    }

    /**
     * Returns a string representation of the reply status type. 
     *
     * @return String reply status type description
     *
     * @since ACT 0.1
     */
    public String toString() {
		String[] types = {"NO_EXCEPTION",
						  "USER_EXCEPTION",
						  "SYSTEM_EXCEPTION",
						  "LOCATION_FORWARD",};
		return types[value];
    }
}
