/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT %Version%
 *
 */

package EDU.rutgers.CORBA.IOP;

import java.io.InputStreamReader;
import java.io.ByteArrayInputStream;
import java.io.Reader;

import ORG.omg.CORBA.Streamable;

import EDU.rutgers.act.presentation.TypeReader;
import EDU.rutgers.act.presentation.TypeWriter;

import EDU.rutgers.CORBA.presentation.TransferSyntaxFactory;

import EDU.rutgers.CORBA.IIOP.ProfileBody;

/**
 * [CORBA 2.0 Specification 10-15]
 *
 * @since ACT 0.1
 */
public final class IOR implements Streamable
{
    /**
     * @since ACT 0.1
     */
    public String type_id_;

    /**
     * @since ACT 0.1
     */
    public TaggedProfile[] profiles_;

    /**
     * @since ACT 0.1
     */
    public IOR() {
    }

    /**
     * @param String type_id
     * @param TaggedProfile[] profiles
     *
     * @since ACT 0.1
     */
    public IOR(String type_id, TaggedProfile[] profiles) {
		type_id_ = type_id;
		profiles_ = profiles;
    }

    /**
     * Retrieve the IIOP ProfileBody if the tag is TAG_INTERNET_IOP.
     * From the IIOP ProfileBody, the user could get the host name, port, and object_key.
     * The user should make the transfer syntax consistent.
     *
     * @param String the transfer syntax used such as "CDR"
     *
     * @return ProfileBody If any error happens, return null.
     *
     * @see EDU.rutgers.CORBA.IIOP.ProfileBody
     * 
     * @since ACT 0.1
     */
    public ProfileBody getIIOPProfile(String transferSyntax) {
        int len = profiles_.length;
        for (int i = len; i-- != 0; ) {
            if (profiles_[len - i - 1].tag_ == TAG_INTERNET_IOP.value) {
                ByteArrayInputStream stream = new ByteArrayInputStream(profiles_[0].profile_data_);
                TypeReader in = TransferSyntaxFactory.createTypeReader(transferSyntax, 
                    new InputStreamReader(stream));

                ProfileBody profile = new ProfileBody();
                try {
                    profile.read(in);
                } catch (java.io.IOException e) {
                    // ??? what should we do?
                }
                return profile;
            }
        }
        return null;
    }

    /**
     * Read from the TypeReader stream.
     *
     * @exception java.io.IOException If any IO error occurs
     *
     * @see ORG.omg.CORBA.Streamable#read
     * @see EDU.rutgers.act.presentation.TypeReader
     *
     * @since ACT 0.1
     */
    public void read(TypeReader in) 
        throws java.io.IOException {
        type_id_ = in.readString();
        int len = in.readULong();
        profiles_ = new TaggedProfile[len];
        for (int i = len; i-- != 0;) {
            profiles_[len - i - 1] = new TaggedProfile();
            profiles_[len - i - 1].read(in);
        }
    }

    /**
     * Write to the TypeWriter stream.
     *
     * @exception java.io.IOException If any IO error occurs
     *
     * @see ORG.omg.CORBA.Streamable#write
     * @see EDU.rutgers.act.presentation.TypeWriter
     *
     * @since ACT 0.1
     */
    public void write(TypeWriter out)
        throws java.io.IOException {
        out.writeString(type_id_);
        int len = profiles_.length;
        out.writeULong(len);
        for (int i = len; i-- != 0;) {
            profiles_[len - i - 1].write(out);
        }
    }

// Object
/*
    public boolean equals(Object o) {
		if((o != null) && (o instanceof IOR)) {
            if (o == this) {
                return true;
            }
			IOR ior = (IOR)o;
			if(!type_id_.equals(ior.type_id_) ||
				profiles_.length != ior.profiles_.length) {
				return false;
			}
			for(int index = 0; index < profiles_.length; index++) {
				if(!profiles_[index].equals(ior.profiles_[index])) {
					return false;
				}
			}
			return true;
		}
		return false;
    }
*/

    /**
     * Returns a string representation. 
     *
     * @return String description
     *
     * @since ACT 0.1
     */
    public String toString() {
		StringBuffer buf = new StringBuffer("IOP:IOR\n\t");
        int len = profiles_.length;
		for(int i = len; --i != 0; ) {
			buf.append(profiles_[len - i].toString());
		}
		return String.valueOf(buf);
    }
}
