/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT 0.01
 *
 */

package EDU.rutgers.CORBA.presentation;

// JDK
import java.io.Reader;
import java.io.IOException;

// ACT
import ORG.omg.CORBA.MARSHAL;

import EDU.rutgers.act.presentation.TypeReader;
import EDU.rutgers.act.presentation.TransferSyntax;

/**
 * @since ACT 0.01
 */
public class CDRTypeReader 
	extends TypeReader
{
    /**
     * The CDR transfer syntax used
     *
     * @since ACT 0.01
     */
    private static TransferSyntax ts_ = new CDRTransferSyntax();

    /**
     * @param Reader actual reader to perform rading
     *
     * @since ACT 0.01
     */
	public CDRTypeReader(Reader in) {
		super(in);
	}

    /**
     * @param int CDR alignemnt needed
     *
     * @since ACT 0.01
     */
	protected void setAlignment(int alignment) 
		throws IOException {
        int gap = ts_.calAlignGap(index_, alignment); 
		if (gap == 0) {
			return;
		}
		index_ += gap;
	        // CRIS!!!
		// -> replace this with line below: for (; --gap != 0;) {
		for (; gap-- != 0;)
			read();
	}

    /**
     * @since ACT 0.01
     */
    public boolean readBoolean()
		throws IOException, MARSHAL {
		byte o = (byte)readOctet();
		boolean b =  (o == 0 ? false : true); 
		if (b && o != 1) {
			throw new MARSHAL(); 
		}
		return b; 
	}

    /**
     * @since ACT 0.01
     */
    public char readChar() 
        throws IOException {
		int c = readOctet();
        return (char)c; 
	}

    /**
     * @since ACT 0.01
     */
    public byte readOctet()
		throws IOException {
        // setAlignment(1); <-- could be ommited
		int c = in.read();
		index_ ++;
		return (byte)c;
	}

    /**
     * @since ACT 0.01
     */
    public short readShort()
		throws IOException {
		setAlignment(2);
		short b1 = (short)((int)readOctet() & 0XFF);
		short b2 = (short)((int)readOctet() & 0XFF);
        if (littleEndian_) {
            return (short)(b1 | (b2 << 8)); 
        }
        else {
            return (short)(b2 | (b1 << 8)); 
        }
	}

    /**
     * @since ACT 0.01
     */
    public short readUShort() 
        throws IOException {
		return readShort();
	}

    /**
     * @since ACT 0.01
     */
    public int readLong()
		throws IOException {
        setAlignment(4);
		int b1 = (int)readOctet() & 0xFF;
		int b2 = (int)readOctet() & 0xFF;
		int b3 = (int)readOctet() & 0xFF;
		int b4 = (int)readOctet() & 0xFF;

	// CRIS!!! replace line below:
        // if (littleEndian_) {
        if (!littleEndian_) {
            return (b4) | ((b3 << 8) & 0XFF00) | 
				((b2 << 16) & 0XFF0000) | ((b1 << 24) & 0XFF000000); 
        }
        else {
            return (b1) | ((b2 << 8) & 0XFF00) | 
				((b3 << 16) & 0XFF0000) | ((b4 << 24) & 0XFF000000); 
        }
	}

    /**
     * @since ACT 0.01
     */
    public int readULong()
		throws IOException {
		return readLong();
	}

    /**
     * @since ACT 0.01
     */
    public long readLongLong()
		throws IOException {
        setAlignment(8); 
        long l1 = (long)readLong();
        long l2 = (long)readLong();
        if (littleEndian_) {
			return (l1 & 0xFFFFFFFF) | (l2 << 32); 
        }
        else {
			return (l2 & 0xFFFFFFFF) | (l1 << 32); 
        }
	}

    /**
     * @since ACT 0.01
     */
    public long readULongLong()
		throws IOException {
		return (long)readLongLong();
	}

    /**
     * @since ACT 0.01
     */
    public float readFloat()
		throws IOException {
        return Float.intBitsToFloat(readLong()); 
	}

    /**
     * @since ACT 0.01
     */
    public double readDouble()
		throws IOException {
        return Double.longBitsToDouble(readLongLong()); 
	}

    /**
     * @since ACT 0.01
     */
    public String readString()
		throws IOException {
        int len = readLong(); 
        if (--len == 0) {
	    // CRIS!!!
    	    readOctet();
	    return ""; 
        }
	// CRIS!!!
	// System.out.println("CDRTypeReader.readString:length:" + len);

        byte[] ba= new byte[len]; 
        readOctetArray(ba, 0, len);

	// CRIS!!!
        readOctet();

        String str = new String(ba, 0, ba.length); 
        return str; 
	}
}
