/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT %Version%
 *
 */

package EDU.rutgers.act.protocol;

import java.io.IOException;

import ORG.omg.CORBA.Streamable;
import EDU.rutgers.act.presentation.TypeReader; 
import EDU.rutgers.act.presentation.TypeWriter; 
import EDU.rutgers.act.endpoint.Endpoint;
/**
 * Abstract conception of Message. Further functionalities should be added. 
 *
 * @see Streamable
 *
 * @since ACT 0.1
 */
public interface Message extends Streamable
{
    /**
     * Return the corresponding message SUB type.
     *
     * @since ACT 0.1
     */
    public abstract int getType();

    /**
     * Return the transaction id. A transaction involves several
     * messages exchange, all of them wearing the same transaction id.
     *
     * @since ACT 0.1
     */
    public abstract int getTransactionId();

    /**
     * Return the corresponding message endpoint. (Required by francu@caip.rutgers.edu)
     *
     * @since ACT 0.1
     */
    public abstract Endpoint getEndpoint();

    /**
     * @see Streamable
     * @see Streamable#read
     *
     * @since ACT 0.1
     */
    public abstract void read(TypeReader in) 
		throws java.io.IOException;

    /**
     * @see Streamable
     * @see Streamable#write
     *
     * @since ACT 0.1
     */
    public abstract void write(TypeWriter out)
		throws java.io.IOException;
}

