/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT %Version%
 *
 */

package ORG.omg.CORBA;

/**
 * Standard Exception base class
 * <p>[CORBA 2.0 Specification 3-33]
 *
 * @see CompletionStatus
 *
 * @see UNKNOWN
 * @see BAD_PARAM
 * @see NO_MEMORY
 * @see IMP_LIMIT
 * @see COMM_FAILURE
 * @see INV_OBJREF
 * @see NO_PERMISSION
 * @see INTERNAL
 * @see MARSHAL
 * @see INITIALIZE
 * @see NO_IMPLEMENT
 * @see BAD_TYPECODE
 * @see BAD_OPERATION
 * @see NO_RESOURCES
 * @see NO_RESPONSE
 * @see PERSIST_STORE
 * @see BAD_INV_ORDER
 * @see TRANSIENT
 * @see FREE_MEM
 * @see INV_IDENT
 * @see INV_FLAG
 * @see INTF_REPOS
 * @see BAD_CONTEXT
 * @see OBJ_ADAPTER
 * @see DATA_CONVERSION
 * @see OBJECT_NOT_EXIST
 *
 * @see UserException
 *
 * @since ACT 0.1
 */
public abstract class SystemException
    extends RuntimeException
{
    /**
     * Designates the subcategory of the exception, 
     * the assignment of values to the minor code is left to each ORB implementation
     *
     * @since ACT 0.1
     */
    public int minor_;

    /**
     * Status
     *
     * @see CompletionStatus 
     *
     * @since ACT 0.1
     */
    public CompletionStatus completed_;

    /**
     * minor = 0
     * <p>CompletionStatus = COMPLETED_MAYBE
     *
     * @since ACT 0.1
     */
    protected SystemException() {       
        this(0, CompletionStatus.COMPLETED_MAYBE);
    }

    /**
     * minor = 0
     * <p>CompletionStatus = COMPLETED_MAYBE
     *
     * @param String exception reason
     *
     * @since ACT 0.1
     */
    protected SystemException(String s) {
        this(s, 0, CompletionStatus.COMPLETED_MAYBE); 
    }

    /**
     * @param int minor code
     * @param CompletionStatus status
     *
     * @since ACT 0.1
     */
    protected SystemException(int minor, CompletionStatus completed) {
		super();
        init(minor, completed); 
    }

    /**
     * @param int minor code
     * @param CompletionStatus status
     * @param String exception reason
     *
     * @since ACT 0.1
     */
    protected SystemException(String s, int minor, CompletionStatus completed) {
        super(s);
        init(minor, completed); 
    }

    private void init(int minor, CompletionStatus completed) {
        minor_ = minor;
		completed_ = completed; 
    }

    /**
     * Returns a string representation.
     *
     * @return String description
     *
     * @since ACT 0.1
     */
    public String toString() {
        return  super.toString() + 
                "CORBA::SystemException " +
			    completed_.toString() + 
			    "\tminor code: " + minor_;
    }
}
